package #
Date::Manip::Offset::off017;
# Copyright (c) 2008-2025 Sullivan Beck.  All rights reserved.
# This program is free software; you can redistribute it and/or modify it
# under the same terms as Perl itself.

# This file was automatically generated.  Any changes to this file will
# be lost the next time 'tzdata' is run.
#    Generated on: Sun Jun  1 17:32:44 EDT 2025
#    Data version: tzdata2025b
#    Code version: tzcode2025b

# This module contains data from the zoneinfo time zone database.  The original
# data was obtained from the URL:
#    ftp://ftp.iana.org/tz

use strict;
use warnings;
require 5.010000;

our ($VERSION);
$VERSION='6.98';
END { undef $VERSION; }

our ($Offset,%Offset);
END {
   undef $Offset;
   undef %Offset;
}

$Offset        = '+01:00:00';

%Offset        = (
   0 => [
      'europe/brussels',
      'europe/belgrade',
      'europe/budapest',
      'europe/prague',
      'europe/berlin',
      'europe/malta',
      'europe/rome',
      'europe/vienna',
      'europe/zurich',
      'africa/ndjamena',
      'africa/tunis',
      'europe/tirane',
      'europe/warsaw',
      'africa/lagos',
      'europe/madrid',
      'africa/algiers',
      'europe/paris',
      'europe/andorra',
      'europe/gibraltar',
      'europe/dublin',
      'africa/casablanca',
      'africa/ceuta',
      'africa/el_aaiun',
      'etc/gmt+1',
      'n',
      'africa/sao_tome',
      'africa/tripoli',
      'europe/vilnius',
      'europe/lisbon',
      'europe/london',
      'europe/kaliningrad',
      'europe/sofia',
      'europe/tallinn',
      'europe/athens',
      'europe/chisinau',
      'europe/minsk',
      'europe/riga',
      'europe/simferopol',
      'europe/kyiv',
      ],
   1 => [
      'europe/lisbon',
      'europe/london',
      'atlantic/madeira',
      'atlantic/canary',
      'atlantic/faroe',
      'africa/casablanca',
      'africa/el_aaiun',
      'africa/windhoek',
      'atlantic/azores',
      'africa/algiers',
      'africa/ceuta',
      'europe/dublin',
      'europe/gibraltar',
      'europe/paris',
      'europe/brussels',
      'europe/madrid',
      ],
);

1;
