package #
Date::Manip::Offset::off271;
# Copyright (c) 2008-2025 Sullivan Beck.  All rights reserved.
# This program is free software; you can redistribute it and/or modify it
# under the same terms as Perl itself.

# This file was automatically generated.  Any changes to this file will
# be lost the next time 'tzdata' is run.
#    Generated on: Sun Jun  1 17:32:44 EDT 2025
#    Data version: tzdata2025b
#    Code version: tzcode2025b

# This module contains data from the zoneinfo time zone database.  The original
# data was obtained from the URL:
#    ftp://ftp.iana.org/tz

use strict;
use warnings;
require 5.010000;

our ($VERSION);
$VERSION='6.98';
END { undef $VERSION; }

our ($Offset,%Offset);
END {
   undef $Offset;
   undef %Offset;
}

$Offset        = '-04:00:00';

%Offset        = (
   0 => [
      'america/puerto_rico',
      'america/glace_bay',
      'america/halifax',
      'america/moncton',
      'america/barbados',
      'america/guyana',
      'america/martinique',
      'america/boa_vista',
      'america/campo_grande',
      'america/cuiaba',
      'america/manaus',
      'america/porto_velho',
      'america/thule',
      'america/santiago',
      'atlantic/bermuda',
      'america/la_paz',
      'america/caracas',
      'america/goose_bay',
      'america/santo_domingo',
      'etc/gmt-4',
      'd',
      'america/coyhaique',
      'america/asuncion',
      'america/grand_turk',
      'america/punta_arenas',
      'america/eirunepe',
      'america/rio_branco',
      'atlantic/stanley',
      'america/argentina/san_luis',
      'america/santarem',
      'america/argentina/catamarca',
      'america/argentina/la_rioja',
      'america/argentina/mendoza',
      'america/argentina/rio_gallegos',
      'america/argentina/san_juan',
      'america/argentina/tucuman',
      'america/argentina/ushuaia',
      'america/argentina/cordoba',
      'america/argentina/jujuy',
      'america/argentina/salta',
      'america/miquelon',
      'america/argentina/buenos_aires',
      'america/cayenne',
      'america/montevideo',
      'antarctica/palmer',
      ],
   1 => [
      'america/new_york',
      'america/toronto',
      'america/havana',
      'america/detroit',
      'america/iqaluit',
      'america/kentucky/louisville',
      'america/indiana/indianapolis',
      'america/indiana/marengo',
      'america/indiana/vevay',
      'america/indiana/vincennes',
      'america/indiana/winamac',
      'america/grand_turk',
      'america/port-au-prince',
      'america/kentucky/monticello',
      'america/indiana/petersburg',
      'america/cancun',
      'america/eirunepe',
      'america/lima',
      'america/bogota',
      'america/guayaquil',
      'america/rio_branco',
      'america/jamaica',
      'america/indiana/tell_city',
      'america/santo_domingo',
      'america/coyhaique',
      'america/punta_arenas',
      'america/santiago',
      ],
);

1;
